# -*- coding: utf-8 -*-
########################################################################################################################
# Copyright (c) 2019, University of Osnabrueck                                                                         #
#   All rights reserved.                                                                                               #
#                                                                                                                      #
#   Redistribution and use in source and binary forms, with or without modification, are permitted provided that the   #
#   following conditions are met:                                                                                      #
#                                                                                                                      #
#   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following#
#       disclaimer.                                                                                                    #
#                                                                                                                      #
#   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the       #
#       following disclaimer in the documentation and/or other materials provided with the distribution.               #
#                                                                                                                      #
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, #
#   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE  #
#   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR    #
#   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  #
#   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   #
#   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           #
########################################################################################################################

"""
This programs helps extracting the maximal signal strength (= path gain) at a given distance. This can be used to select
a threshold signal strength for network simulation. In the future, this will be replaced by a normalization of the
values in RaLaNS.
"""


import zipfile
import src.util.parseResFile as pf
import sys
from src.util.functions import parseConfigFile
import numpy as np
import itertools

if len(sys.argv) !=3:
    print("Two arguments required! Usage: ./extractSignalStrengthThreshold.py path/to/outputzipfile distanceinmeters")
zipf = sys.argv[1]
radius = float(sys.argv[2])
zf = zipfile.ZipFile(zipf, 'r')

rf = 'result.txt'
cf = 'config.cfg'
resfile = zf.open(rf)
configfile = zf.open(cf)

config = parseConfigFile(configfile, isZip=True)     

rectype = 0
trantype = 0
for i, l in enumerate(resfile):
    if i == 0:
        trantype = int(l[0])
    if i == 1:
        rectype = int(l[0])
        break
resfile.close()
resfile = zf.open(rf)   

print rectype, trantype
transmitters = np.array(pf.getTransmitters(resfile, isZip=True))

maxfind = -100
for transmitter in transmitters:
    resfile = zf.open(rf)
    res=pf.parseResFile(resfile, transmitter, stepsize=config["stepSize"], requestedLayer=None, isZip=True, setinf=False, suppressoutput=True)
    receivers=[]
    vals=[]
    if rectype == 4: #list, street
        receivers = np.reshape(res[3],(-1,3))
        vals=res[0][0]
        
    elif rectype == 2: #area
        ylen, xlen = np.shape(res[0])
        xs = np.arange(xlen)*config["stepSize"]+config["borders"][0]
        ys = np.arange(ylen)*config["stepSize"]+config["borders"][1]
        
        receivers = np.array(list(itertools.product(xs,ys,[config["coverageLevel"]])))
        vals = np.array(np.transpose(res[0]).flatten())
    else:
        print("Receiver type not supported, is {}, should be 2 (area) or 4 (list, streets).".format(rectype))
        sys.exit()    
    mask = np.sum((receivers - transmitter)**2,1) >= radius**2
    maxcand = np.max(vals[mask])
    maxfind = max(maxfind, maxcand)
    print maxcand, maxfind    

